package net.wanji.opt.common;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import lombok.extern.slf4j.Slf4j;
import net.wanji.opt.dto.export.utils.DynamicExportLaneDataElement;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.net.URLEncoder;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
* @author duanruiming
* @date 2024/06/13 9:35
*/
@Slf4j
public class ExcelExportUtils {

    public static <T> void exportExcel(HttpServletResponse response, String startStr, String endStr,
                                       List<T> dataList, String sheetName, Class<T> clazz) throws Exception {
        // 文件名 sheet名称添加日期
        String fileName = sheetName.concat(startStr).concat("~").concat(endStr)
                .replaceAll(":", " ").replaceAll(" ", "-").concat(".xlsx"); // 注意文件扩展名应为.xlsx，因为你设置了xlsx的MIME类型

        // 设置响应头
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setHeader("Content-Disposition", "attachment; filename=" + URLEncoder.encode(fileName, "UTF-8").replaceAll("\\+", "%20"));

        // 使用EasyExcel进行导出
        try (ServletOutputStream outputStream = response.getOutputStream()) {
            // 解决服务器导出文件没数据问题 核心这个配置开始内存处理模式 inMemory少量数据
            //ExcelWriter excelWriter = EasyExcel.write(outputStream, clazz).inMemory(Boolean.TRUE).build();
            ExcelWriter excelWriter = EasyExcel.write(outputStream, clazz).build();
            WriteSheet writeSheet = EasyExcel.writerSheet(sheetName).build();

            // 填充数据
            excelWriter.write(dataList, writeSheet);

            // 刷新并关闭流，不需要重复关闭outputStream
            excelWriter.finish();
        } catch (Exception e) {
            log.error("{}导出excel异常：", fileName, e);
            throw new Exception("导出excel异常：" + e.getMessage(), e);
        }
    }

    /**
     * 动态导出excel
     * @param response
     * @param startStr
     * @param endStr
     * @param dataList
     * @param sheetName
     * @throws Exception
     */
    public static void exportExcel(HttpServletResponse response, String startStr, String endStr,
                                   List<DynamicExportLaneDataElement> dataList, String sheetName) throws Exception {
        // 文件名 sheet名称添加日期
        String fileName = sheetName.concat(startStr).concat("~").concat(endStr)
                .replaceAll(":", " ").replaceAll(" ", "-").concat(".xlsx"); // 注意文件扩展名应为.xlsx，因为你设置了xlsx的MIME类型

        // 设置响应头
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setHeader("Content-Disposition", "attachment; filename=" + URLEncoder.encode(fileName, "UTF-8").replaceAll("\\+", "%20"));

        // 使用EasyExcel进行导出
        try (ServletOutputStream outputStream = response.getOutputStream()) {
            // 手动提取表头（确保dataList不为空）
            if (dataList.isEmpty()) {
                log.error("{}导出excel异常：dataList为空", fileName);
                return;
            }
            DynamicExportLaneDataElement de = dataList.get(0);
            List<String> headerList = de.getFields().stream()
                    .map(DynamicExportLaneDataElement.DynamicField::getFieldName)
                    .collect(Collectors.toList());

            // 构建表头结构（List<List<String>>）
            List<List<String>> head = headerList.stream()
                    .map(Collections::singletonList)
                    .collect(Collectors.toList());

            // 配置WriteSheet并绑定表头
            WriteSheet writeSheet = EasyExcel.writerSheet(sheetName)
                    .head(head)  // 关键：通过head()方法绑定表头
                    .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy()) // 自动列宽
                    .build();

            // 提取数据行（List<List<Object>>）
            List<List<Object>> data = dataList.stream()
                    .map(element -> element.getFields().stream()
                            .map(DynamicExportLaneDataElement.DynamicField::getFieldValue)
                            .collect(Collectors.toList()))
                    .collect(Collectors.toList());

            // 直接写入数据（无需单独写表头）
            EasyExcel.write(outputStream)
                    .autoCloseStream(true)
                    .sheet(sheetName)
                    .head(head)
                    .doWrite(data);

        } catch (Exception e) {
            log.error("{}导出excel异常：", fileName, e);
            throw new Exception("导出excel异常：" + e.getMessage(), e);
        }
    }

}
